#!/bin/bash
#
# STE
#
# ReleaseDate: 20200914
# Created by Cartor on 2019-06-13.
# Copyright 2019 Splashtop Inc. All rights reserved.
#

#----------------common function start-----------------------#


function exceCMD() {
    cmd="${1}"
    echo "${cmd}" | sh
    #echo "${cmd}"
    #echo "${cmd}" >> ~/uninstaller.log
}

function tarCopyFile() {
    _DIR=$(pwd)

    SOURCE="$(dirname "$1")"
    DESTINATION="$2"
    FILE="$(basename "$1")"

    echo "Copying file to ${DESTINATION}"
    cd "${SOURCE}"
    sudo tar cf - "${FILE}" | (cd "${DESTINATION}"; sudo tar xfp -)
    cd "${_DIR}"
}

function vercomp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

function setDeployCode() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>DeployCode</key>
        <string>REPALCE_STRING</string>
        <key>DeployTeamNameCache</key>
        <string></string>
        <key>DeployTeamOwnerCache</key>
        <string></string>
        <key>LastDeployCode</key>
        <string></string>
        <key>TeamCode</key>
        <string></string>
        <key>TeamCodeInUse</key>
        <string></string>
EOF
}

function setShowDeployLoginWarning() {
    FILENAME="${1}"

    ShowDeployLoginWarning="true"
    if [ "${2// }" == "0" ]; then
        ShowDeployLoginWarning="false"
    fi

    sed "s/REPALCE_STRING/${ShowDeployLoginWarning}/g" <<- EOF >> "${FILENAME}"
        <key>ShowDeployLoginWarning</key>
        <REPALCE_STRING/>
EOF
}

function setComputerName() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>HostName</key>
        <string>REPALCE_STRING</string>
EOF
}

function setPermissionProtectionOption() {
    FILENAME="${1}"
    REQUEST_PERMISSION="${2}"
    EnablePermissionProtection=""

    if [ -z "${REQUEST_PERMISSION// }" ]; then
        return
    fi

    EnablePermissionProtection="${REQUEST_PERMISSION// }"

    if [ ! -z "${EnablePermissionProtection// }" ]; then
        sed "s/REPALCE_STRING/${EnablePermissionProtection}/g" <<- EOF >> "${FILENAME}"
        <key>EnablePermissionProtection</key>
        <integer>REPALCE_STRING</integer>
EOF
    fi    
}

function setSecurityOption() {
    FILENAME="${1}"
    SECURITY_OPTION="${2}"

    EnableSecurityCodeProtection=""
    EnableOSCredential=""

    if [ "$SECURITY_OPTION" == "0" ]; then
        EnableSecurityCodeProtection="false"
        EnableOSCredential="false"
    fi

    if [ "$SECURITY_OPTION" == "1" ]; then
        EnableSecurityCodeProtection="true"
        EnableOSCredential="false"
    fi

    if [ "$SECURITY_OPTION" == "2" ]; then
        EnableSecurityCodeProtection="false"
        EnableOSCredential="true"
    fi
    
    if [ ! -z "${EnableSecurityCodeProtection// }" ]; then
        sed "s/REPALCE_STRING/${EnableSecurityCodeProtection}/g" <<- EOF >> "${FILENAME}"
        <key>EnableSecurityCodeProtection</key>
        <REPALCE_STRING/>
EOF
    fi

    if [ ! -z "${EnableOSCredential// }" ]; then
        sed "s/REPALCE_STRING/${EnableOSCredential}/g" <<- EOF >> "${FILENAME}"
        <key>EnableOSCredential</key>
        <REPALCE_STRING/>
EOF
    fi    
}

function setInitSecurityCode() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>init_security_code</key>
        <string>REPALCE_STRING</string>
EOF
}

function setLegacyConnectionLoopbackOnly() {
    FILENAME="${1}"

    cat <<- EOF >> "${FILENAME}"
        <key>LegacyConnectionLoopbackOnly</key>
        <true/>
EOF
}

function setHideTrayIcon() {
    FILENAME="${1}"

    cat <<- EOF >> "${FILENAME}"
        <key>HideTrayIcon</key>
        <true/>
EOF
}

function setDefaultClientDeviceName() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>DefaultClientDeviceName</key>
        <string>REPALCE_STRING</string>
EOF
}

function setShowStreamerUI() {
    FILENAME="${1}"

    cat <<- EOF >> "${FILENAME}"
        <key>FirstTimeClose</key>
        <false/>
        <key>FirstTimeLogin</key>
        <false/>
        <key>PermissionAlert</key>
        <false/>
EOF
}

function setEnableLanConnection() {
    FILENAME="${1}"

    cat <<- EOF >> "${FILENAME}"
        <key>EnableLanConnection</key>
        <false/>
EOF
}

function setCommonDict() {
    FILENAME="${1}"
    STREAMER_TYPE="${2}"
    STREAMER="${3}"

    if [ "$STREAMER_TYPE" == "0" ]; then
        cat <<- EOF >> "${FILENAME}"
    <key>Common</key>
    <dict>
        <key>HidePreferenceDomainSelection</key>
        <true/>
        <key>EulaAccepted</key>
        <true/>
    </dict>
EOF
    else
        sed "s/REPALCE_STRING/${STREAMER_TYPE}/g" <<- EOF >> "${FILENAME}"
    <key>Common</key>
    <dict>
        <key>HidePreferenceDomainSelection</key>
        <true/>
        <key>EulaAccepted</key>
        <true/>
        <key>StreamerType</key>
        <integer>REPALCE_STRING</integer>
    </dict>
EOF
    fi
}

function setStreamerTypeDict() {
    FILENAME="${1}"
    STREAMER_TYPE="${2}"
    STREAMER="${3}"

    sed "s/REPALCE_STRING/${STREAMER}/g" <<- EOF >> "${FILENAME}"
    <key>REPALCE_STRING</key>
    <dict>
EOF

    if [ "$STREAMER_TYPE" == "0" ]; then
        cat <<- EOF >> "${FILENAME}"
        <key>ShowDeployMode</key>
        <true/>
        <key>SplashtopAccount</key>
        <string></string>
EOF
    fi

    if [ "$STREAMER_TYPE" == "1" ]; then
        cat <<- EOF >> "${FILENAME}"
        <key>ShowDeployMode</key>
        <true/>
        <key>SplashtopAccount</key>
        <string></string>
EOF
    fi

    if [ "$STREAMER_TYPE" == "2" ]; then
        cat <<- EOF >> "${FILENAME}"
        <key>FirstTimeLogin</key>
        <false/>
        <key>BackendConnected</key>
        <true/>
        <key>ClientCertificateData</key>
        <data>
        </data>
        <key>CustomizeTeamCode</key>
        <string></string>
        <key>FirstTimeLogin</key>
        <false/>
        <key>IsNewUUIDScheme</key>
        <true/>
        <key>RelayConnected</key>
        <true/>
EOF
    fi
}

function setPlistByStreamerType() {
    FILENAME="${1}"
    STREAMER_TYPE="${2}"
    STREAMER=""

    if [ "$STREAMER_TYPE" == "0" ]; then
        STREAMER="STP"
    fi

    if [ "$STREAMER_TYPE" == "1" ]; then
        STREAMER="STB"
    fi

    if [ "$STREAMER_TYPE" == "2" ]; then
        STREAMER="STE"
    fi

    cat <<- EOF > "${FILENAME}"
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
    <key>UniversalSetting</key>
    <true/>
EOF

    setCommonDict "${FILENAME}" "${STREAMER_TYPE// }" "${STREAMER// }"
    setStreamerTypeDict "${FILENAME}" "${STREAMER_TYPE// }" "${STREAMER// }"
}

function setBackendAddress() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>BackendAddress</key>
        <string>REPALCE_STRING</string>
EOF
}

function setBackendAccount() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>SplashtopAccount</key>
        <string>REPALCE_STRING</string>
EOF
}

function setBackendPassword() {
    FILENAME="${1}"

    sed "s/REPALCE_STRING/${2}/g" <<- EOF >> "${FILENAME}"
        <key>SplashtopPassword</key>
        <string>REPALCE_STRING</string>
EOF
}

function restartApp() {
    APPNAME="${1}"
    ps axc -Ouser | grep -i "${APPNAME}" | awk '{print $1}' | xargs kill
    sleep 2
    open -b `osascript -e "id of app \"${APPNAME}\""`
}

function removeSoundDriver() {
    KextInstallPath=("/Library/Extensions" "/System/Library/Extensions")
    KextName=("SplashtopSoundDriver.kext" "Soundflower.kext")

    for _kextPath in "${KextInstallPath[@]}" ; do
        for _kextName in "${KextName[@]}" ; do
            is_unload=true
            _kext="${_kextPath}/${_kextName}"
            _plist="${_kext}/Contents/Info.plist"

            BundleIdentifier=""
            Version=""

            [ -e "${_kext}" ] || continue

            if [ -e "${_plist}" ]; then
                #CFBundleIdentifier
                #CFBundleShortVersionString
                echo "${_plist}"

                BundleIdentifier=$(/usr/libexec/PlistBuddy -c "Print :CFBundleIdentifier" "${_plist}")
                Version=$(/usr/libexec/PlistBuddy -c "Print :CFBundleShortVersionString" "${_plist}")
            fi

            if [[ "${_kextName}" == "Soundflower.kext" && "${Version}" != "1.6.7" ]]; then
                is_unload=false
            fi

            if [ "$is_unload" = true ]; then
                [ -e "${_kext}" ] && echo "unstall kext... [${_kext}]" ; remove_kext "${_kext}" "${BundleIdentifier}"
            fi
        done
    done
}

function remove_kext() {
    _kext="$1"
    bundle_id="$2"

    if [ -z "${bundle_id}" ]; then
        exceCMD "sudo kextunload ${_kext}"
    else
        exceCMD "sudo kextstat -b \"${bundle_id}\" > /dev/null 2>&1 && sudo kextunload -b \"${bundle_id}\""
    fi

    exceCMD "sudo rm -rf \"${_kext}\""
}

function installSoundDriver() {
    KEXTPATH="${1}"
    KEXTNAME="${2}"
    #SplashtopSoundDriver.kext

    MAJOR_MAC_VERSION=$(sw_vers -productVersion | awk -F '.' '{print $1 "." $2}')
    vercomp "${MAJOR_MAC_VERSION}" "10.14"
    [[ $? -eq 1 ]] && KEXTINSTALLPATH="/Library/Extensions" || KEXTINSTALLPATH="/System/Library/Extensions"

    # copy app to tmp folder
    tarCopyFile "${KEXTPATH}/${KEXTNAME}" "${KEXTINSTALLPATH}"
    
    sudo chown -R root:wheel "${KEXTINSTALLPATH}/${KEXTNAME}"
    sudo chmod -R 755 "${KEXTINSTALLPATH}/${KEXTNAME}"
    sudo kextload "${KEXTINSTALLPATH}/${KEXTNAME}"
    sudo kextcache --clear-staging > /dev/null 2>&1
    sudo kextutil "${KEXTINSTALLPATH}/${KEXTNAME}"
    sudo kextstat | grep "${KEXTNAME%.*}"
}
#----------------common function end-------------------------#

function usage()
{
    echo -e "Usage: `basename $1` [-i input streamer dmg file] [-z STE backend address] [-a splashtop account] [-u splashtop password] ..."
    echo -e "\t-i : input streamer dmg file."
    echo -e "\t-k : input streamer pkg file."
    echo -e "\t-z : STE backend address."
    echo -e "\t-a : STE backend account."
    echo -e "\t-u : STE backend password."
    echo -e "\t-t : to skip install pkg or dmg, to re-config setting."
}

#----------------usage function--------------------------#

CHECK_NEED_DMG_IN="0"
CHECK_NEED_DEPLOY_CODE="0"
CHECK_NEED_PKG_IN="0"
BackendAddress=""
BackendAccount=""
BackendPassword=""
SkippedInstall="0"

#check options
OPTIND=1 # Reset is necessary if getopts was used previously in the script.  It is a good idea to make this local in a function.
while getopts i:k:v:z:a:u:t o
do	case "$o" in
	i)
        DMG_IN="$OPTARG"
        CHECK_NEED_DMG_IN="1"
		;;
    k)
        PKG_IN="$OPTARG"
        CHECK_NEED_PKG_IN="1"
        ;;
    v)
        INSTALL_DRIVER="$OPTARG"
        ;;
    z)
        BackendAddress="${OPTARG// }"
        ;;
    a)
        BackendAccount="${OPTARG// }"
        ;;
    u)
        BackendPassword="${OPTARG// }"
        ;;
    t)
        SkippedInstall="1"
        ;;
	[?])
		echo "invalid option: $1" 1>&2;
		usage "$0"
		exit 1;;
	esac
done
shift $((OPTIND-1)) # Shift off the options and optional --.

if [ "${CHECK_NEED_DMG_IN}" == "0" ] && [ "${CHECK_NEED_PKG_IN}" == "0" ] && [ "${SkippedInstall}" == "0" ]; then
    echo "Please input streamer dmg or pkg file!"
    usage "$0"
    exit 1
fi

#if [ "$CHECK_NEED_DEPLOY_CODE" == "0" ]; then
#echo "No deploy code!"
#usage "$0"
#exit 1
#fi

PRE_INSTALL_PATH="/Users/Shared/SplashtopStreamer"

#write .PreInstall
echo "Inject settings"
mkdir "$PRE_INSTALL_PATH"

if [ "$INSTALL_DRIVER" == "0" ]; then
    NO_DRIVER="$PRE_INSTALL_PATH/.NoDriver"
    touch "$NO_DRIVER"
fi

TEMP_DIR="$(mktemp -d)"
trap "rm -rf $TEMP_DIR" EXIT

PRE_INSTALL="${PRE_INSTALL_PATH}/.PreInstall"
FILENAME="${TEMP_DIR}/.PreInstall.$$"
echo "Writing file ${PRE_INSTALL}"
#set plist header and common dict
# 0: STP, 1: STB, 2: STE
setPlistByStreamerType "${FILENAME}" "2"
    
[ ! -z "${BackendAddress// }" ] && setBackendAddress "${FILENAME}" "${BackendAddress// }"
[ ! -z "${BackendAccount// }" ] && setBackendAccount "${FILENAME}" "${BackendAccount// }"
[ ! -z "${BackendPassword// }" ] && setBackendPassword "${FILENAME}" "${BackendPassword// }"

cat <<- EOF >> "${FILENAME}"
    </dict>
</dict>
</plist>
EOF
sudo cp -r "${FILENAME}" "${PRE_INSTALL}"
rm -rf "${FILENAME}"
#sudo chown -R root:wheel "${PRE_INSTALL}"
sudo chmod -R 755 "${PRE_INSTALL}"

if [ "${SkippedInstall}" == "0" ]; then
    if [ "$CHECK_NEED_DMG_IN" == "1" ]; then
        if [ ! -f "${DMG_IN}" ]; then
            echo "${DMG_IN} is not exist."
            rm -rf "${PRE_INSTALL}" 2>/dev/null || true
            exit 1
        fi

        #mount dmg
        VOLUME=$(hdiutil attach -nobrowse "${DMG_IN}" | awk 'END {print $3}')
        [ -z "${VOLUME}" ] && VOLUME="/Volumes/SplashtopStreamer"
        echo "Mounting dmg file, Volume ${VOLUME}."

        echo "Install silently"
        NORMAL_INSTALLER="${VOLUME}/Splashtop Streamer.pkg"
        HIDDEN_INSTALLER="${VOLUME}/.Splashtop Streamer.pkg"
        if [ -f "${NORMAL_INSTALLER}" ]; then
            sudo installer -pkg "${NORMAL_INSTALLER}" -target /
        else
            sudo installer -pkg "${HIDDEN_INSTALLER}" -target /
        fi

        echo "Unmount dmg. ${VOLUME}"
        hdiutil detach -quiet "${VOLUME}"
    else
        if [ ! -f "${PKG_IN}" ]; then
            echo "${PKG_IN} is not exist."
            rm -rf "${PRE_INSTALL}" 2>/dev/null || true
            exit 1
        fi
        sudo installer -pkg "${PKG_IN}" -target /
    fi
else
    if [ ! -e "/Applications/Splashtop Streamer.app" ]; then
        echo "Error : Splashtop Streamer is not installed."

        rm -rf "${PRE_INSTALL}" 2>/dev/null || true
        exit 1;
    fi

    echo "Skipped install pkg or dmg, to re-config setting."
    restartApp "Splashtop Streamer"
fi

if [ "$INSTALL_DRIVER" == "1" ] && [ "$USER" == "root" ]; then
    os_ver="$(sw_vers -productVersion)"
    IFS='.' read -r -a ver <<< "$os_ver"

    MAJOR_MAC_VERSION="${ver[1]}"

    vercomp "${ver[0]}.${ver[1]}" "10.8"
    if [ $? == 1 ]; then 
        removeSoundDriver
        installSoundDriver "/Applications/Splashtop Streamer.app/Contents/Resources/updated" "SplashtopSoundDriver.kext"
    fi
fi

#echo "Launch Streamer"
#open "/Applications/Splashtop Streamer.app"
echo "Done!"

exit 0

